def format_book(title: str, author: str) -> str:
    """Format a book recommendation.

    Example:
        {"title": "THE HOBBIT", "author": "J.R.R. Tolkien"}

    Example:
        {"title": "THE NAME OF THE WIND", "author": "Patrick Rothfuss"}
    """
    return f"I recommend {title} by {author}."
